package com.winhc.bigdata.spark.ng.dynamic

import org.apache.spark.sql.Row
import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
import org.apache.commons.lang3.StringUtils

/**
 * @author: XuJiakai
 * @date: 2021/6/22 13:49
 */
case class ChangeExtract(rowkey: String
                         , company_id: String
                         , company_name: String
                         , tn: String
                         , update_type: String
                         , old_data: Map[String, String]
                         , new_data: Map[String, String]
                         , change_fields: Seq[String]
                         , biz_date: String
                         , update_time: String
                        )


case class AssociationEntityInfo(keyno: String
                                 , name: String
                                 , risk_level: NgCompanyRiskLevelType.RiskLevelType //变更风险等级
                                 , rta_info: String //描述
                                )

case class RowkeyInfo(rowkey: String, tn: String) {
  def toStr(): String = {
    s"$tn@@$rowkey"
  }
}

case class CompanyDynamicRecord(id: String,
                                association_entity_info: Seq[AssociationEntityInfo],
                                rowkey: String,
                                tn: String,
                                update_type: String,
                                dynamic_info: Map[String, String],
                                agg_detail_text: String,
                                agg_detail_rowkey: Seq[RowkeyInfo],
                                old_record: Map[String, String],
                                new_record: Map[String, String],
                                change_time: String,
                                update_time: String,
                                create_time: String
                               ) {
  def format(): CompanyDynamicRecord = {
    if (id == null) {
      return null
    }
    if (association_entity_info == null || association_entity_info.isEmpty) {
      return null
    }
    val rec = association_entity_info.filter(r => StringUtils.isNotEmpty(r.keyno))
    if (rec.isEmpty) return null

    if (rec.length != association_entity_info.length)
      return CompanyDynamicRecord(id,
        rec,
        rowkey,
        tn,
        update_type,
        dynamic_info,
        agg_detail_text,
        agg_detail_rowkey,
        old_record,
        new_record,
        change_time,
        update_time,
        create_time
      )
    this
  }

  def to_row(): Row = {
    val risk_level_str = association_entity_info.map(_.risk_level).distinct.mkString(",")
    val risk_level_detail = association_entity_info.map(r => s"${r.keyno}@@${r.risk_level}").distinct.mkString(",")
    val agg_detail_rowkey_str: String = if (agg_detail_rowkey == null) null else agg_detail_rowkey.map(_.toStr).mkString(",")
    Row(id
      , association_entity_info.toJson()
      , rowkey
      , tn
      , update_type
      , risk_level_str
      , risk_level_detail
      , dynamic_info.toJson()
      , agg_detail_text
      , agg_detail_rowkey_str
      , old_record.toJson()
      , new_record.toJson()
      , s"${tn}_${change_time}"
      , change_time
      , update_time
      , create_time
      , 0L
      , null
    )
  }
}


